import Polygon from '@/models/segmentation/Polygon'

describe('Polygon', () => {
  it('raises validation error if the number of points % 2 != 0', () => {
    expect(() => {
      const polygon = new Polygon(1, [1, 2, 3, 4, 5, 6, 7], 'id')
    }).toThrow()
  })

  it('can instanciate itself', () => {
    const polygon = new Polygon(1, [1, 2, 3, 4, 5, 6], 'id')
    expect(polygon.id).toBe('id')
    expect(polygon.labelId).toBe(1)
    expect(polygon.numberOfSides).toBe(3)
  })

  it('translate polygon', () => {
    const polygon = new Polygon(1, [1, 2, 3, 4, 5, 6], 'id')
    polygon.translate(1, 2)
    const expectedPoints = [2, 4, 4, 6, 6, 8]
    expect(polygon.flattenedPoints).toEqual(expectedPoints)
  })

  it('cannot be closed if number of sides is less than 3', () => {
    const polygon = new Polygon(1, [1, 2, 3, 4], 'id')
    expect(polygon.canBeClosed()).toBe(false)
  })

  it('can be closed if number of sides is greater than 3', () => {
    const polygon = new Polygon(1, [1, 2, 3, 4, 5, 6], 'id')
    expect(polygon.canBeClosed()).toBe(true)
  })

  it('can add point', () => {
    const polygon = new Polygon(1, [1, 2], 'id')
    polygon.addPoint(3, 4)
    const expectedPoints = [1, 2, 3, 4]
    expect(polygon.flattenedPoints).toEqual(expectedPoints)
  })

  it('can move point', () => {
    const polygon = new Polygon(1, [1, 2, 3, 4], 'id')
    polygon.movePoint(1, 5, 6)
    const expectedPoints = [1, 2, 5, 6]
    expect(polygon.flattenedPoints).toEqual(expectedPoints)
  })

  it('can remove point', () => {
    const polygon = new Polygon(1, [1, 2, 3, 4, 5, 6, 7, 8], 'id')
    polygon.removePoint(1)
    const expectedPoints = [1, 2, 5, 6, 7, 8]
    expect(polygon.flattenedPoints).toEqual(expectedPoints)
  })

  it('cannot remove point if the number of sides are less than 3', () => {
    const polygon = new Polygon(1, [1, 2, 3, 4, 5, 6], 'id')
    polygon.removePoint(1)
    const expectedPoints = [1, 2, 3, 4, 5, 6]
    expect(polygon.flattenedPoints).toEqual(expectedPoints)
  })

  it('can insert point', () => {
    const polygon = new Polygon(1, [1, 2, 3, 4, 5, 6], 'id')
    polygon.insertPoint(7, 8, 1)
    const expectedPoints = [1, 2, 7, 8, 3, 4, 5, 6]
    expect(polygon.flattenedPoints).toEqual(expectedPoints)
  })
})
